/*****************************************************************************/
/*
  animate.c: animate a data file.
*/
/*****************************************************************************/
/* This version flips X and angles to get lighting right */
#define XFLIP -
#define XXFLIP 
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "drawstuff.h" /* ODE Graphics stuff */
#include "drawstuff-cga.h" /* CGA stuff to make things clearer */

#include "../useful/mrdplot/mrdplot.h"
#include "main.h"
#include "main2.h"

/*****************************************************************************/

enum channels {
TIME=0,
SHOULDER_A,
ELBOW_A,
SHOULDERD,
ELBOWD,
ELBOW_X,
ELBOW_Z,
WRIST_X,
WRIST_Z,
SHOULDER_TORQUE,
ELBOW_TORQUE,
N_CHANNELS
};

#define UPPER_ARM_LENGTH (0.3f) // length of link
#define UPPER_ARM_WIDTH (0.05f)    // width of link

#define FOREARM_LENGTH (0.3f)     // length of link
#define FOREARM_WIDTH (0.05f)    // width of link

/*****************************************************************************/

SIM sim;

MRDPLOT_DATA *d;

int channels[N_CHANNELS];

double shoulder_z = 1.0;

/*****************************************************************************/
/*****************************************************************************/

double lookup_value( MRDPLOT_DATA *d, int point, int channel )
{
  if ( channels[channel] < 0 || channels[channel] >= d->n_channels )
    {
      fprintf( stderr, "Bad channel: %d %d\n", channel, channels[channel] );
      exit( -1 );
    }
  return d->data[point*d->n_channels + channels[channel]];
}

/********************************************************************/
// start(): set things up

static void start()
{
  // set up view point
  static float xyz[3] = {0.0f,1.0f,1.0f};
  static float hpr[3] = {-90.0000f,0.0000f,0.0000f};
  dsSetViewpoint (xyz,hpr);
}

/********************************************************************/
// called when a key pressed

static void command (int cmd)
{
  // don't handle user input yet.
  dsPrint ("received command %d (`%c')\n",cmd,cmd);
}

/*****************************************************************************/

static void display (int pause)
{
  static int index = 0;
  float f;
  float center[3];
  float R[12];
  int i;
  float upper_arm_sides[3]
    = { UPPER_ARM_WIDTH, UPPER_ARM_WIDTH, UPPER_ARM_LENGTH };
  float forearm_sides[3]
    = { FOREARM_WIDTH, FOREARM_WIDTH, FOREARM_LENGTH };

  /* UPPER ARM (assume shoulder is at 0, 0) */
  center[XX] = XFLIP ( lookup_value( d, index, ELBOW_X ) )/2;
  center[YY] = 0;
  center[ZZ] = ( lookup_value( d, index, ELBOW_Z ) )/2 + shoulder_z;

  for ( i = 0; i < 12; i++ )
    R[i] = 0;
  R[0] = cos( lookup_value( d, index, SHOULDER_A ) );
  R[10] = R[0];
  R[2] = XXFLIP sin( lookup_value( d, index, SHOULDER_A ) );
  R[8] = - R[2];

  dsSetTexture (DS_WOOD);
  dsSetColor (0,0,1);
  dsDrawBox( center, R, upper_arm_sides );


  /* Torso */
  center[XX] = XFLIP (lookup_value( d, index, ELBOW_X )
		+ lookup_value( d, index, WRIST_X ))/2;
  center[YY] = 0;
  center[ZZ] = (lookup_value( d, index, ELBOW_Z )
		+ lookup_value( d, index, WRIST_Z ))/2 + shoulder_z;

  for ( i = 0; i < 12; i++ )
    R[i] = 0;
  R[0] = cos( lookup_value( d, index, SHOULDER_A )
	      + lookup_value( d, index, ELBOW_A ) );
  R[10] = R[0];
  R[2] = XXFLIP sin( lookup_value( d, index, SHOULDER_A )
		     + lookup_value( d, index, ELBOW_A ) );
  R[8] = - R[2];

  dsSetTexture (DS_WOOD);
  dsSetColor (1,0,0);
  dsDrawBox( center, R, forearm_sides );

  /*
  printf( "%g %g %g %g %g %g\n",
	  lookup_value( d, index, SHOULDER_A ),
	  lookup_value( d, index, ELBOW_A ),
	  lookup_value( d, index, ELBOW_X ),
	  lookup_value( d, index, ELBOW_Z ),
	  lookup_value( d, index, WRIST_X ),
	  lookup_value( d, index, WRIST_Z ) );
  */

  /* Delay */
  for( f = 0.0; f < 2000000.0; f += 1.0 )
    ;

  index += 1;

  if ( index > d->n_points )
    {
      printf( ".\n" );
      index = 0;
    }
}

/*****************************************************************************/
/********************************************************************/

int main (int argc, char **argv)
{
  char *filename;
  int i;
  dsFunctions fn; 

  if ( argc > 1 )
    filename = strdup( argv[1] );
  else
    filename = last_data();
  
  printf( "Reading: %s\n", filename );

  d = read_mrdplot( filename );

  for( i = 0; i < N_CHANNELS; i++ )
    channels[i] = -1;

  channels[TIME] = find_channel( "time", d );
  channels[SHOULDER_A] = find_channel( "shoulder_angle", d );
  channels[ELBOW_A] = find_channel( "elbow_angle", d );
  channels[SHOULDERD] = find_channel( "shoulderd", d );
  channels[ELBOWD] = find_channel( "elbowd", d );
  channels[ELBOW_X] = find_channel( "elbow_x", d );
  channels[ELBOW_Z] = find_channel( "elbow_z", d );
  channels[WRIST_X] = find_channel( "wrist_x", d );
  channels[WRIST_Z] = find_channel( "wrist_z", d );
  channels[SHOULDER_TORQUE] = find_channel( "shoulder_torque", d );
  channels[ELBOW_TORQUE] = find_channel( "elbow_torque", d );

  /*
  for( i = 0; i < N_CHANNELS; i++ )
    printf( "%d %d\n", i, channels[i] );
  */

  // setup pointers to drawstuff callback functions
  fn.version = DS_VERSION; 
  fn.start = &start; 
  fn.step = &display; 
  fn.command = &command; 
  fn.stop = 0; 
#ifdef WIN32
  fn.path_to_textures = "C:/cga/kdc/sim5/useful/drawstuff-windows/textures";
#else
  fn.path_to_textures = "/home/cga/teaching/kdc/sim5/useful/drawstuff-linux/textures";
#endif

  // do display
  dsSimulationLoop( argc, argv, /* command line arguments */
                    352, 288, /* window size */
                    &fn ); /* callback info */

  return 0;
}

/********************************************************************/
