/*****************************************************************************/
/*
  data.c: save data from simulation
*/
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../useful/mrdplot/mrdplot.h"

#include "main.h"
#include "main2.h"

/*****************************************************************************/

extern SIM sim;

/*****************************************************************************/

enum channels {
MRDPLOT_TIME=0,
MRDPLOT_SHOULDER,
MRDPLOT_ELBOW,
MRDPLOT_SHOULDERD,
MRDPLOT_ELBOWD,
MRDPLOT_ELBOW_X,
MRDPLOT_ELBOW_Z,
MRDPLOT_WRIST_X,
MRDPLOT_WRIST_Z,
MRDPLOT_SHOULDER_TORQUE,
MRDPLOT_ELBOW_TORQUE,
N_CHANNELS
};

#define MAX_N_POINTS 100000

int n_points = 0;
float data[MAX_N_POINTS*N_CHANNELS];

char *names[N_CHANNELS];
char *units[N_CHANNELS];

/*****************************************************************************/

void init_data( SIM *s )
{
  int i;

  for( i = 0; i < N_CHANNELS; i++ )
    {
      names[i] = "xxx";
      units[i] = "xxx";
    }

  names[MRDPLOT_TIME] = "time";
  names[MRDPLOT_SHOULDER] = "shoulder_angle";
  names[MRDPLOT_ELBOW] = "elbow_angle";
  names[MRDPLOT_SHOULDERD] = "shoulderd";
  names[MRDPLOT_ELBOWD] = "elbowd";
  names[MRDPLOT_ELBOW_X] = "elbow_x";
  names[MRDPLOT_ELBOW_Z] = "elbow_z";
  names[MRDPLOT_WRIST_X] = "wrist_x";
  names[MRDPLOT_WRIST_Z] = "wrist_z";
  names[MRDPLOT_SHOULDER_TORQUE] = "shoulder_torque";
  names[MRDPLOT_ELBOW_TORQUE] = "elbow_torque";

  units[MRDPLOT_TIME] = "s";
  units[MRDPLOT_SHOULDER] = "r";
  units[MRDPLOT_ELBOW] = "r";
  units[MRDPLOT_SHOULDERD] = "r/s";
  units[MRDPLOT_ELBOWD] = "r/s";
  units[MRDPLOT_ELBOW_X] = "m";
  units[MRDPLOT_ELBOW_Z] = "m";
  units[MRDPLOT_WRIST_X] = "m";
  units[MRDPLOT_WRIST_Z] = "m";
  units[MRDPLOT_SHOULDER_TORQUE] = "Nm";
  units[MRDPLOT_ELBOW_TORQUE] = "Nm";
  
  /*
  for( i = 0; i < N_CHANNELS; i++ )
    {
      printf( "%d %s %s\n", i, names[i], units[i] );
    }
  */
}

/*****************************************************************************/

int save_data( SIM *s )
{
  static int myindex = 0;

  if ( n_points >= MAX_N_POINTS )
    return 0;

  data[myindex + MRDPLOT_TIME] = (float) (s->time);
  data[myindex + MRDPLOT_SHOULDER] = (float) (s->angle[SHOULDER]);
  data[myindex + MRDPLOT_ELBOW] = (float) (s->angle[ELBOW]);
  data[myindex + MRDPLOT_SHOULDERD] = (float) (s->angled[SHOULDER]);
  data[myindex + MRDPLOT_ELBOWD] = (float) (s->angled[ELBOW]);
  data[myindex + MRDPLOT_ELBOW_X] = (float) (s->elbow[XX]);
  data[myindex + MRDPLOT_ELBOW_Z] = (float) (s->elbow[ZZ]);
  data[myindex + MRDPLOT_WRIST_X] = (float) (s->wrist[XX]);
  data[myindex + MRDPLOT_WRIST_Z] = (float) (s->wrist[ZZ]);
  data[myindex + MRDPLOT_SHOULDER_TORQUE] = (float) (s->torque[SHOULDER]);
  data[myindex + MRDPLOT_ELBOW_TORQUE] = (float) (s->torque[ELBOW]);

  myindex += N_CHANNELS;
  n_points++;

  return 0;
}

/*****************************************************************************/

int write_ascii_file( char *filename )
{
  FILE *stream;
  int i;
  static int myindex = 0;

  stream = fopen( filename, "w" );
  if ( stream == NULL )
    {
      fprintf( stderr, "Can't open %s for write.\n", filename );
      exit( -1 );
    }
  for ( i = 0; i < n_points; i++ )
    {
      fprintf( stream, "%g ", data[myindex + MRDPLOT_TIME] );
      fprintf( stream, "%g ", data[myindex + MRDPLOT_SHOULDER] );
      fprintf( stream, "%g ", data[myindex + MRDPLOT_ELBOW] );
      fprintf( stream, "%g ", data[myindex + MRDPLOT_SHOULDERD] );
      fprintf( stream, "%g ", data[myindex + MRDPLOT_ELBOWD] );
      fprintf( stream, "%g ", data[myindex + MRDPLOT_SHOULDER_TORQUE] );
      fprintf( stream, "%g ", data[myindex + MRDPLOT_ELBOW_TORQUE] );
      fprintf( stream, "\n" );
      myindex += N_CHANNELS;
    }
  fclose( stream );

  return 0;
}

/*****************************************************************************/

int write_the_mrdplot_file( SIM *s )
{
  MRDPLOT_DATA *d; 

  d = malloc_mrdplot_data( 0, 0 );
  d->filename = generate_file_name();
  d->n_channels = N_CHANNELS;
  d->n_points = n_points;
  d->total_n_numbers = d->n_channels*d->n_points;
  d->frequency = (float) (1/s->time_step);
  d->data = data;
  d->names = names;
  d->units = units;

  write_mrdplot_file( d );

  d->filename[0] = 'a';
  write_ascii_file( d->filename );

  return 0;
}

/*****************************************************************************/
