/******************************************************************/
/* 
Defines, typedefs, and structs
*/
/******************************************************************/
/*
*/
/******************************************************************/

/* Status flags */
#define OK 0
#define CRASHED 1
 
/* These are handy constants. XX rather than X to avoid math.h conflicts */
#define XX 0
#define YY 1
#define ZZ 2
#define XYZ 3

#define N_DOFS 2 // number of joints.
#define N_X 4 // number of states.
#define N_U 2 // number of controls.

// Degrees of freedom
#define SHOULDER 0
#define ELBOW 1

#ifdef WIN32
#define M_PI 3.14159
#endif

/******************************************************************/
/******************************************************************/
/******************************************************************/
/******************************************************************/
/******************************************************************/
/******************************************************************/
// Overall simulation structure.

typedef struct {

  // *** COMMON VARIABLES ACROSS ALL SIMULATIONS

  double time_step; // controller time step    
  double time;      // current time in simulation
  int status;       // Are we CRASHED? 

  double state[N_X];

  // Joint angles: points to state
  double *angle;

  // Angular velocities: points to state
  double *angled;

  // torques
  double torque[N_U];

  // Location of various body parts. */
  double shoulder[XYZ];
  double elbow[XYZ];
  double wrist[XYZ];

  // Linear velocities
  double shoulderd[XYZ];
  double elbowd[XYZ];
  double wristd[XYZ];

  // *** KINEMATIC AND DYNAMIC PARAMETERS

  /* Put limits in?
double the_state_mins[N_STATE_DIMENSIONS] = { -1.5*M_PI, -M_PI, -12.0, -12.0 };
double the_state_maxs[N_STATE_DIMENSIONS] = {  0.0,  M_PI,  12.0, 12.0 };
double the_action_mins[N_ACTION_DIMENSIONS] = { -100.0, -100.0 };
double the_action_maxs[N_ACTION_DIMENSIONS] = {  100.0,  100.0 };
  */

  double upper_arm_length;
  double upper_arm_cm;
  double upper_arm_mass;
  double upper_arm_I;
  double forearm_length;
  double forearm_cm;
  double forearm_mass;
  double forearm_I;

  double gravity;
  double viscous_friction[N_DOFS];
  double force_forearm_fx;
  double force_forearm_fz;
  double force_forearm_r;
  double force_upper_arm_fx;
  double force_upper_arm_fz;
  double force_upper_arm_r;

  // stuff to speed up dynamics
  double l0cm_m0;
  double l1cm_m1;
  double g_l1cm_m1;
  double l1cm_l1cm_m1;
  double l0_l1cm_m1;
  double l0cm_m0_p_l0_m1;
  double g_l0cm_m0_p_l0_m1;
  double expr2;
  double expr3;
  double total_mass;

  // *** CONTROLLER **************

  // desired angles
  double angle_d[N_DOFS];

  // desired angular velocities
  double angled_d[N_DOFS];

  // desired task positions
  double wrist_d[XYZ];

  // desired task velocities
  double wristd_d[XYZ];

  // *** MISCELLANEOUS ***********************

  // simulation duration
  double duration;

} SIM;

/******************************************************************/
